import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:designflow_material_addon/core/key_constants.dart';
import 'package:designflow_material_addon/designflow_material_addon.dart';
import 'package:designflow_material_addon/src/widgets/appbar.dart';
import 'package:designflow_material_addon/src/widgets/bottombar.dart';
import 'package:designflow_material_addon/src/widgets/floating_action_button.dart';
import 'package:designflow_material_addon/src/widgets/icon_button.dart';
import 'package:designflow_material_addon/src/widgets/range_slider.dart';
import 'package:designflow_material_addon/src/widgets/slider.dart';
import 'package:designflow_material_addon/src/widgets/time_picker.dart';
import 'package:flutter/material.dart';

/// Material Button
class MaterialButtonAdapter extends WidgetAdapter {
  const MaterialButtonAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialButton(
        nodeState: state,
        type: state.node.getAttributes[MaterialDBKeys.materialButtonType],
        textStyle: state.node.getAttributes[DBKeys.textStyle],
        value: state.node.getAttributes[DBKeys.value],
        isFullWidth: state.node.getAttributes[DBKeys.isFullWidth],
        fill: state.node.getAttributes[DBKeys.fill],
        faIcon: state.node.getAttributes[DBKeys.faIcon],
        featherIcon: state.node.getAttributes[DBKeys.featherIcon],
        materialIcon: state.node.getAttributes[DBKeys.icon],
      );

  MaterialButtonAdapter.create() : this();
}

/// Floating Action Button
class FloatingActionButtonAdapter extends WidgetAdapter {
  const FloatingActionButtonAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWFloatingActionButton(
        nodeState: state,
        type: state.node.getAttributes[MaterialDBKeys.floatingActionButtonType],
        textStyle: state.node.getAttributes[DBKeys.textStyle],
        value: state.node.getAttributes[DBKeys.value],
        fill: state.node.getAttributes[DBKeys.fill],
        faIcon: state.node.getAttributes[DBKeys.faIcon],
        featherIcon: state.node.getAttributes[DBKeys.featherIcon],
        materialIcon: state.node.getAttributes[DBKeys.icon],
      );

  FloatingActionButtonAdapter.create() : this();
}

/// Icon Button
class IconButtonAdapter extends WidgetAdapter {
  const IconButtonAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialIconButton(
        state: state,
        fill: state.node.getAttributes[DBKeys.fill],
        faIcon: state.node.getAttributes[DBKeys.faIcon],
        featherIcon: state.node.getAttributes[DBKeys.featherIcon],
        materialIcon: state.node.getAttributes[DBKeys.icon],
      );

  IconButtonAdapter.create() : this();
}

/// Date Picker
class DatePickerAdapter extends WidgetAdapter {
  const DatePickerAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialDatePicker(
        nodeState: state,
        dayOfWeekStyle: state.node.getAttributes[DBKeys.textStyle],
        dayStyle: state.node.getAttributes[DBKeys.textStyle],
      );

  DatePickerAdapter.create() : this();
}

/// Time Picker
class TimePickerAdapter extends WidgetAdapter {
  const TimePickerAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialTimePicker(
        state: state,
      );

  TimePickerAdapter.create() : this();
}

/// Slider Picker
class SliderAdapter extends WidgetAdapter {
  const SliderAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialSlider(
        state: state,
        fill: state.node.getAttributes[DBKeys.fill],
        min: state.node.getAttributes[DBKeys.min],
        max: state.node.getAttributes[DBKeys.max],
        formatType: state.node.getAttributes[DBKeys.formatType],
      );

  SliderAdapter.create() : this();
}

/// Range Slider Picker
class RangeSliderAdapter extends WidgetAdapter {
  const RangeSliderAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialRangeSlider(
        state: state,
        fill: state.node.getAttributes[DBKeys.fill],
        min: state.node.getAttributes[DBKeys.min],
        max: state.node.getAttributes[DBKeys.max],
        formatType: state.node.getAttributes[DBKeys.formatType],
      );

  RangeSliderAdapter.create() : this();
}

/// AppBar
class AppBarAdapter extends WidgetAdapter {
  const AppBarAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialAppBar(
        nodeState: state,
        fill: state.node.getAttributes[DBKeys.fill],
        title: state.node.getAttributes[DBKeys.value],
        textStyle: state.node.getAttributes[DBKeys.textStyle],
        children: state.node.children ?? [],
      );

  AppBarAdapter.create() : this();
}

/// BottomBar
class BottomBarAdapter extends WidgetAdapter {
  const BottomBarAdapter();
  @override
  Widget toWidget({
    required final BuildContext context,
    required final WidgetState state,
  }) =>
      OpenWMaterialBottomBar(
        nodeState: state,
        backgroundColor: state.node.getAttributes[DBKeys.fill],
        children: state.node.children ?? [],
      );

  BottomBarAdapter.create() : this();
}
