import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:designflow_material_addon/core/floating_action_button_types.dart';
import 'package:uuid/uuid.dart';

import '../../core/key_constants.dart';
import '../../core/material_button_types.dart';

/// Material Button
class MaterialButtonDefaultAttributes extends DefaultAttributesAdapter {
  const MaterialButtonDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.textStyle: const FTextStyle(
            fill: FFill(levels: [FFillElement(color: 'ffffff', stop: 0)])),
        DBKeys.icon: 'plus',
        DBKeys.featherIcon: null,
        DBKeys.lineIcon: null,
        DBKeys.faIcon: null,
        DBKeys.isFullWidth: false,
        DBKeys.fill: const FFill(paletteStyle: 'Accent'),
        DBKeys.value: const StaticTextTypeInput(value: 'Continue'),
        MaterialDBKeys.materialButtonType: MaterialButtonTypes.elevated,
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  MaterialButtonDefaultAttributes.create() : this();
}

/// Floating Action Button
class FloatingActionButtonDefaultAttributes extends DefaultAttributesAdapter {
  const FloatingActionButtonDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.textStyle: const FTextStyle(
            fill: FFill(levels: [FFillElement(color: 'ffffff', stop: 0)])),
        DBKeys.icon: 'plus',
        DBKeys.featherIcon: null,
        DBKeys.lineIcon: null,
        DBKeys.faIcon: null,
        DBKeys.fill: const FFill(paletteStyle: 'Accent'),
        DBKeys.value: const StaticTextTypeInput(value: 'Continue'),
        MaterialDBKeys.floatingActionButtonType:
            FloatingActionButtonTypes.regular,
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  FloatingActionButtonDefaultAttributes.create() : this();
}

/// Floating Action Button
class IconButtonDefaultAttributes extends DefaultAttributesAdapter {
  const IconButtonDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.icon: 'plus',
        DBKeys.featherIcon: null,
        DBKeys.lineIcon: null,
        DBKeys.faIcon: null,
        DBKeys.fill: const FFill(paletteStyle: 'Text / Primary'),
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  IconButtonDefaultAttributes.create() : this();
}

/// Date Picker
class DatePickerDefaultAttributes extends DefaultAttributesAdapter {
  const DatePickerDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.textStyle: const FTextStyle(
          textStyleModel: 'Paragraph',
          fill: FFill(
            paletteStyle: 'Text / Primary',
          ),
        ),
        DBKeys.workflows: [
          WorkflowEntity(
            trigger: Trigger.onDateChanged,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                cubitID: null,
                variableID: null,
                stateID: null,
              ),
            ],
          ),
          /*WorkflowEntity(
            trigger: Trigger.onStartDateChanged,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                variableID: '',
              ),
            ],
          ),
          WorkflowEntity(
            trigger: Trigger.onEndDateChanged,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                variableID: '',
              ),
            ],
          ),*/
        ],
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  DatePickerDefaultAttributes.create() : this();
}

/// Time Picker
class TimePickerDefaultAttributes extends DefaultAttributesAdapter {
  const TimePickerDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{};

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  TimePickerDefaultAttributes.create() : this();
}

/// Calendar Picker
class CalendarPickerDefaultAttributes extends DefaultAttributesAdapter {
  const CalendarPickerDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.workflows: [
          WorkflowEntity(
            trigger: Trigger.onChange,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                cubitID: null,
                variableID: null,
                stateID: null,
              ),
              AssingVariableFromElement(
                id: const Uuid().v1(),
                cubitID: null,
                variableID: null,
                stateID: null,
              ),
            ],
          ),
        ]
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  CalendarPickerDefaultAttributes.create() : this();
}

/// Slider
class SliderDefaultAttributes extends DefaultAttributesAdapter {
  const SliderDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.min: 0,
        DBKeys.max: 2399,
        DBKeys.fill: const FFill(paletteStyle: 'Accent'),
        DBKeys.formatType: FormatType.hourAndMinutes,
        DBKeys.workflows: [
          WorkflowEntity(
            trigger: Trigger.onChange,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                cubitID: null,
                variableID: null,
                stateID: null,
              ),
            ],
          ),
        ]
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  SliderDefaultAttributes.create() : this();
}

/// Range Slider
class RangeSliderDefaultAttributes extends DefaultAttributesAdapter {
  const RangeSliderDefaultAttributes();

  @override
  Map<String, dynamic> get get => <String, dynamic>{
        DBKeys.min: const StaticTypeInput<double>(
          value: 0,
          type: FGeneralTypeEnum.double,
        ),
        DBKeys.max: const StaticTypeInput<double>(
          value: 0,
          type: FGeneralTypeEnum.double,
        ),
        DBKeys.fill: const FFill(paletteStyle: 'Accent'),
        DBKeys.formatType: FormatType.hourAndMinutes,
        DBKeys.workflows: [
          WorkflowEntity(
            trigger: Trigger.onMinValueChanged,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                cubitID: null,
                variableID: null,
                stateID: null,
              ),
            ],
          ),
          WorkflowEntity(
            trigger: Trigger.onMaxValueChanged,
            actions: [
              AssingVariableFromElement(
                id: const Uuid().v1(),
                cubitID: null,
                variableID: null,
                stateID: null,
              ),
            ],
          ),
        ]
      };

  @override
  List<String> requiredAttributes(String nodeType) => <String>[];

  RangeSliderDefaultAttributes.create() : this();
}
