import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:designflow_material_addon/core/node_type.dart';
import 'package:designflow_material_addon/designflow_material_addon.dart';
import 'package:designflow_material_addon/src/intrinsic_states/states.dart';

/// Material Button
class MaterialButtonOpenNode extends CNode {
  MaterialButtonOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.materialButton,
          name: name ?? 'Material Button',
          parentID: parentID,
          intrinsicState: const MaterialButtonIntrinsicStates(),
          defaultAttributes: const MaterialButtonDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const MaterialButtonAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const MaterialButtonDefaultAttributes()
                  .requiredAttributes(MaterialNType.materialButton),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      MaterialButtonOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      MaterialButtonOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(
    String widgetType,
    Map<String, dynamic> json,
    DynamicAttributes parser,
    ActionMapper actionMapper,
  ) {
    if (widgetType != MaterialNType.materialButton) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return MaterialButtonOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'MaterialButtonOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'MaterialButtonOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/// Floating Action Button
class FloatingActionButtonOpenNode extends CNode {
  FloatingActionButtonOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.floatingActionButton,
          name: name ?? 'Floating Action Button',
          parentID: parentID,
          intrinsicState: const FloatingActionButtonIntrinsicStates(),
          defaultAttributes: const FloatingActionButtonDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const FloatingActionButtonAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const FloatingActionButtonDefaultAttributes()
                  .requiredAttributes(MaterialNType.floatingActionButton),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      FloatingActionButtonOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      FloatingActionButtonOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.floatingActionButton) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return FloatingActionButtonOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'FloatingActionButtonOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'FloatingActionButtonOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/// Icon Button
class IconButtonOpenNode extends CNode {
  IconButtonOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.iconButton,
          name: name ?? 'Floating Action Button',
          parentID: parentID,
          intrinsicState: const IconButtonIntrinsicStates(),
          defaultAttributes: const IconButtonDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const IconButtonAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const IconButtonDefaultAttributes()
                  .requiredAttributes(MaterialNType.iconButton),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      IconButtonOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      IconButtonOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.iconButton) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return IconButtonOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'IconButtonOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'IconButtonOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/// Icon Button
class DatePickerOpenNode extends CNode {
  DatePickerOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.datePicker,
          name: name ?? 'Date Picker',
          parentID: parentID,
          intrinsicState: const DatePickerIntrinsicStates(),
          defaultAttributes: const DatePickerDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const DatePickerAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const DatePickerDefaultAttributes()
                  .requiredAttributes(MaterialNType.datePicker),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      DatePickerOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      DatePickerOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.datePicker) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return DatePickerOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'DatePickerOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'DatePickerOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/// Time Button
class TimePickerOpenNode extends CNode {
  TimePickerOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.timePicker,
          name: name ?? 'Time Picker',
          parentID: parentID,
          intrinsicState: const TimePickerIntrinsicStates(),
          defaultAttributes: const TimePickerDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const TimePickerAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const TimePickerDefaultAttributes()
                  .requiredAttributes(MaterialNType.timePicker),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      TimePickerOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      TimePickerOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.timePicker) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return TimePickerOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'TimePickerOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'TimePickerOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/// Slider
class SliderOpenNode extends CNode {
  SliderOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.slider,
          name: name ?? 'Slider',
          parentID: parentID,
          intrinsicState: const SliderIntrinsicStates(),
          defaultAttributes: const SliderDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const SliderAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const SliderDefaultAttributes()
                  .requiredAttributes(MaterialNType.slider),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      SliderOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      SliderOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.slider) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return SliderOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() => 'SliderOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'SliderOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/// Range Slider
class RangeSliderOpenNode extends CNode {
  RangeSliderOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.rangeSlider,
          name: name ?? 'Range Slider',
          parentID: parentID,
          intrinsicState: const RangeSliderIntrinsicStates(),
          defaultAttributes: const RangeSliderDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const RangeSliderAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const RangeSliderDefaultAttributes()
                  .requiredAttributes(MaterialNType.rangeSlider),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      RangeSliderOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      RangeSliderOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.rangeSlider) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return RangeSliderOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'RangeSliderOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'RangeSliderOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}

/*
/// AppBar
class AppBarOpenNode extends CNode {
  AppBarOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.appBar,
          name: name ?? 'AppBar',
          parentID: parentID,
          intrinsicState: const AppBarIntrinsicStates(),
          defaultAttributes: const AppBarDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const AppBarAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          isAppBar: true,
          requiredAttributes: requiredAttributes ??
              const AppBarDefaultAttributes()
                  .requiredAttributes(MaterialNType.appBar),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      AppBarOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      AppBarOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.appBar) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return AppBarOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() => 'AppBarOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'AppBarOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}
*/

/*
/// BottomBar
class BottomBarOpenNode extends CNode {
  BottomBarOpenNode({
    required final NodeID id,
    final NodeID? parentID,
    final CNode? child,
    final List<CNode>? children,
    final String? name,
    final String? description,
    final double? childOrder,
    final Map<String, dynamic>? attributes,
    final RectProperties? rectProperties,
    final DateTime? updatedAt,
    final PageID? pageID,
    final NodeID? stabilID,
    final PageID? componentID,
    final List<CNode>? componentChildren,
    final bool? isLocked,
    final List<String>? requiredAttributes,
  }) : super(
          id: id,
          type: MaterialNType.bottomBar,
          name: name ?? 'BottomBar',
          parentID: parentID,
          intrinsicState: const BottomBarIntrinsicStates(),
          defaultAttributes: const BottomBarDefaultAttributes().get,
          attributes: attributes ?? {},
          rectProperties: rectProperties ?? CNode.defaultRProperties,
          adapter: const BottomBarAdapter(),
          description: description,
          childOrder: childOrder ?? 0,
          child: child,
          children: children,
          updatedAt: updatedAt ?? DateTime.now(),
          pageID: pageID ?? '',
          stabilID: stabilID,
          componentID: componentID,
          isLocked: isLocked ?? false,
          requiredAttributes: requiredAttributes ??
              const BottomBarDefaultAttributes()
                  .requiredAttributes(MaterialNType.bottomBar),
        );

  @override
  CNode copyWith({
    NodeID? id,
    CNode? text,
    CNode? icon,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      BottomBarOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child ?? this.child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
        requiredAttributes: requiredAttributes ?? this.requiredAttributes,
      );

  @override
  CNode copyWithOutChild({
    NodeID? id,
    NodeID? parentID,
    CNode? child,
    List<CNode>? children,
    String? name,
    String? description,
    double? childOrder,
    Map<String, dynamic>? attributes,
    RectProperties? rectProperties,
    DateTime? updatedAt,
    PageID? pageID,
    NodeID? stabilID,
    PageID? componentID,
    List<CNode>? componentChildren,
    bool? isLocked,
    List<String>? requiredAttributes,
  }) =>
      BottomBarOpenNode(
        id: id ?? this.id,
        parentID: parentID ?? this.parentID,
        child: child,
        children: children ?? this.children,
        name: name ?? this.name,
        description: description ?? this.description,
        childOrder: childOrder ?? this.childOrder,
        attributes: attributes ?? getAttributes,
        rectProperties: rectProperties ?? getRectProperties,
        pageID: pageID ?? this.pageID,
        stabilID: stabilID ?? this.stabilID,
        componentID: componentID ?? this.componentID,
        isLocked: isLocked ?? this.isLocked,
      );

  static fromJson(String widgetType, Map<String, dynamic> json,
      DynamicAttributes parser, ActionMapper actionMapper) {
    if (widgetType != MaterialNType.bottomBar) {
      throw Exception('Invalid widget type');
    }

    final attributes = <String, dynamic>{};
    if (json['properties'] != null) {
      for (final entry in (json['properties'] as Map<String, dynamic>)
          .entries
          .where((e) => e.value != null && e.key != 'key')) {
        attributes[entry.key] =
            parser.fromJson(entry.key, entry.value, actionMapper);
      }
    }

    final rectProperties = RectProperties.fromJson(json['rect_properties']);

    return BottomBarOpenNode(
      id: json['id'],
      name: json['name'],
      description: json['description'],
      parentID: json['parent_id'],
      attributes: attributes,
      rectProperties: rectProperties,
      updatedAt: DateTime.parse(json['updated_at']),
      childOrder: json['child_order'] != null
          ? (json['child_order'] as num).toDouble()
          : null,
      pageID: json['page_id'],
      stabilID: json['stabil_id'],
      componentID: json['component_id'],
      isLocked: json['is_locked'],
    );
  }

  @override
  String toString() =>
      'BottomBarOpenNode { id: $id, name: $name, type: $type }';

  String toStringExtended() =>
      'BottomBarOpenNode { id: $id, name: $name, type: $type, parentID: $parentID, attributes: $getAttributes, childOrder: $childOrder, child: $child, children: $children }';
}
*/