import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:flutter/material.dart';

class OpenWMaterialBottomBar extends StatefulWidget {
  const OpenWMaterialBottomBar({
    super.key,
    required this.nodeState,
    required this.backgroundColor,
    required this.children,
  });

  final WidgetState nodeState;
  final FFill backgroundColor;
  final List<CNode> children;

  @override
  State<OpenWMaterialBottomBar> createState() => _OpenWMaterialBottomBarState();
}

class _OpenWMaterialBottomBarState extends State<OpenWMaterialBottomBar> {
  static const nodeOverride = NodeOverrideExecuter();

  var selectedIndex = 0;

  void _handleOnChanged(int index) {
    runWorkflows(
      node: widget.nodeState.node,
      trigger: Trigger.onChange,
      context: context,
      loop: widget.nodeState.loop,
      inputValue: index,
    );
    setState(() {
      selectedIndex = index;
    });
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuildersForNodes(
      node: widget.nodeState.node,
      builder: (context, state, stylesState, themeMode) {
        final backgroundColor0 = widget.backgroundColor.getColor(
          context,
          widget.nodeState.loop,
        );

        final childn = nodeOverride.executeChildren(
            context, widget.nodeState, widget.children);

        return NavigationBar(
          backgroundColor: backgroundColor0,
          selectedIndex: selectedIndex,
          destinations: childn.length >= 2
              ? childn
              : [
                  const NavigationDestination(
                    icon: Icon(Icons.home),
                    label: 'Home',
                  ),
                  const NavigationDestination(
                    icon: Icon(Icons.search),
                    label: 'Search',
                  ),
                  const NavigationDestination(
                    icon: Icon(Icons.favorite),
                    label: 'Favorites',
                  ),
                  const NavigationDestination(
                    icon: Icon(Icons.person),
                    label: 'Profile',
                  ),
                ],
          onDestinationSelected: _handleOnChanged,
        );
      },
    );
  }
}
