import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:flutter/material.dart';

import '../../core/material_button_types.dart';

class OpenWChip extends StatelessWidget {
  const OpenWChip({
    super.key,
    required this.nodeState,
    required this.type,
    required this.textStyle,
    required this.isFullWidth,
    required this.value,
    required this.fill,
    required this.faIcon,
    required this.featherIcon,
    required this.materialIcon,
  });

  final WidgetState nodeState;
  final MaterialButtonTypes type;
  final FGeneralTypeInput value;
  final String? materialIcon;
  final String? faIcon;
  final String? featherIcon;
  final FTextStyle textStyle;
  final FFill fill;
  final bool isFullWidth;

  void _handleOnPressed(BuildContext context) {
    runWorkflows(
      node: nodeState.node,
      trigger: Trigger.onTap,
      context: context,
      loop: nodeState.loop,
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuildersForNodes(
      node: nodeState.node,
      builder: (context, state, stylesState, themeMode) {
        final backgroundColor = fill.getColor(
          context,
          nodeState.loop,
        );

        final child = Row(
          mainAxisSize: isFullWidth ? MainAxisSize.max : MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            if (featherIcon != null)
              OpenWFeatherIcon(
                nodeState: nodeState,
                icon: featherIcon!,
                width: const FSize(
                  size: '20',
                  sizeTablet: '20',
                  sizeDesktop: '20',
                ),
                fill: textStyle.fill,
              )
            else if (faIcon != null)
              OpenWFontAwesome(
                nodeState: nodeState,
                icon: faIcon!,
                width: const FSize(
                  size: '20',
                  sizeTablet: '20',
                  sizeDesktop: '20',
                ),
                fill: textStyle.fill,
              )
            else
              OpenWIcon(
                nodeState: nodeState,
                icon: materialIcon ?? 'plus',
                width: const FSize(
                  size: '20',
                  sizeTablet: '20',
                  sizeDesktop: '20',
                ),
                fill: textStyle.fill,
              ),
            TextBuilder(
              nodeState: nodeState,
              textStyle: textStyle,
              value: value,
            ),
          ],
        );

        return IgnorePointer(
          ignoring: !state.forPlay,
          child: Builder(builder: (context) {
            switch (type) {
              case MaterialButtonTypes.elevated:
                return ElevatedButton(
                  onPressed: () => _handleOnPressed(context),
                  style: ElevatedButton.styleFrom(
                    backgroundColor: backgroundColor,
                  ),
                  child: child,
                );
              case MaterialButtonTypes.filled:
                return FilledButton(
                  onPressed: () => _handleOnPressed(context),
                  style: FilledButton.styleFrom(
                    backgroundColor: backgroundColor,
                  ),
                  child: child,
                );
              case MaterialButtonTypes.outlined:
                return OutlinedButton(
                  onPressed: () => _handleOnPressed(context),
                  style: OutlinedButton.styleFrom(
                    side: BorderSide(
                      color: backgroundColor,
                    ),
                  ),
                  child: child,
                );
              case MaterialButtonTypes.text:
                return TextButton(
                  onPressed: () => _handleOnPressed(context),
                  child: child,
                );
            }
          }),
        );
      },
    );
  }
}
