import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:designflow_material_addon/core/floating_action_button_types.dart';
import 'package:flutter/material.dart';

class OpenWFloatingActionButton extends StatelessWidget {
  const OpenWFloatingActionButton({
    super.key,
    required this.nodeState,
    required this.type,
    required this.textStyle,
    required this.value,
    required this.fill,
    required this.faIcon,
    required this.featherIcon,
    required this.materialIcon,
  });

  final WidgetState nodeState;
  final FloatingActionButtonTypes type;
  final FGeneralTypeInput value;
  final String? materialIcon;
  final String? faIcon;
  final String? featherIcon;
  final FTextStyle textStyle;
  final FFill fill;

  void _handleOnPressed(BuildContext context) {
    runWorkflows(
      node: nodeState.node,
      trigger: Trigger.onTap,
      context: context,
      loop: nodeState.loop,
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuildersForNodes(
      node: nodeState.node,
      builder: (context, state, stylesState, themeMode) {
        final bgColor = fill.getColor(
          context,
          nodeState.loop,
        );

        final iconWidget = (featherIcon != null)
            ? OpenWFeatherIcon(
                nodeState: nodeState,
                icon: featherIcon!,
                width: const FSize(
                  size: '20',
                  sizeTablet: '20',
                  sizeDesktop: '20',
                ),
                fill: textStyle.fill,
              )
            : (faIcon != null)
                ? OpenWFontAwesome(
                    nodeState: nodeState,
                    icon: faIcon!,
                    width: const FSize(
                      size: '20',
                      sizeTablet: '20',
                      sizeDesktop: '20',
                    ),
                    fill: textStyle.fill,
                  )
                : OpenWIcon(
                    nodeState: nodeState,
                    icon: materialIcon ?? 'plus',
                    width: const FSize(
                      size: '20',
                      sizeTablet: '20',
                      sizeDesktop: '20',
                    ),
                    fill: textStyle.fill,
                  );

        return IgnorePointer(
          ignoring: !state.forPlay,
          child: Builder(
            builder: (context) {
              switch (type) {
                case FloatingActionButtonTypes.small:
                  return FloatingActionButton.small(
                    onPressed: () => _handleOnPressed(context),
                    backgroundColor: bgColor,
                    child: iconWidget,
                  );
                case FloatingActionButtonTypes.regular:
                  return FloatingActionButton(
                    onPressed: () => _handleOnPressed(context),
                    backgroundColor: bgColor,
                    child: iconWidget,
                  );
                case FloatingActionButtonTypes.large:
                  return FloatingActionButton.large(
                    onPressed: () => _handleOnPressed(context),
                    backgroundColor: bgColor,
                    child: iconWidget,
                  );
                case FloatingActionButtonTypes.extended:
                  return FloatingActionButton.extended(
                    onPressed: () => _handleOnPressed(context),
                    backgroundColor: bgColor,
                    label: TextBuilder(
                      nodeState: nodeState,
                      textStyle: textStyle,
                      value: value,
                    ),
                    icon: iconWidget,
                  );
              }
            },
          ),
        );
      },
    );
  }
}
