import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:flutter/material.dart';

class OpenWMaterialIconButton extends StatelessWidget {
  const OpenWMaterialIconButton({
    super.key,
    required this.state,
    required this.fill,
    required this.faIcon,
    required this.featherIcon,
    required this.materialIcon,
  });

  final WidgetState state;
  final String? materialIcon;
  final String? faIcon;
  final String? featherIcon;
  final FFill fill;

  void _handleOnPressed(BuildContext context) {
    runWorkflows(
      node: state.node,
      trigger: Trigger.onTap,
      context: context,
      loop: state.loop,
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuildersForNodes(
      node: state.node,
      builder: (context, appState, pageState, stylesState) {
        final iconWidget = (featherIcon != null)
            ? OpenWFeatherIcon(
                nodeState: state,
                icon: featherIcon!,
                width: const FSize(
                  size: '20',
                  sizeTablet: '20',
                  sizeDesktop: '20',
                ),
                fill: fill,
              )
            : (faIcon != null)
                ? OpenWFontAwesome(
                    nodeState: state,
                    icon: faIcon!,
                    width: const FSize(
                      size: '20',
                      sizeTablet: '20',
                      sizeDesktop: '20',
                    ),
                    fill: fill,
                  )
                : OpenWIcon(
                    nodeState: state,
                    icon: materialIcon ?? 'plus',
                    width: const FSize(
                      size: '20',
                      sizeTablet: '20',
                      sizeDesktop: '20',
                    ),
                    fill: fill,
                  );
        return IgnorePointer(
          ignoring: !appState.forPlay,
          child: IconButton(
            onPressed: () => _handleOnPressed(context),
            icon: iconWidget,
          ),
        );
      },
    );
  }
}
