import 'dart:async';

import 'package:after_layout/after_layout.dart';
import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:flutter/material.dart';

class OpenWMaterialRangeSlider extends StatefulWidget {
  const OpenWMaterialRangeSlider({
    super.key,
    required this.state,
    required this.fill,
    required this.min,
    required this.max,
    required this.formatType,
  });

  final WidgetState state;
  final double min;
  final double max;
  final FFill fill;
  final FormatType formatType;

  @override
  State<OpenWMaterialRangeSlider> createState() => _OpenWMaterialSliderState();
}

class _OpenWMaterialSliderState extends State<OpenWMaterialRangeSlider>
    with AfterLayoutMixin {
  RangeValues values = const RangeValues(0, 1);
  double min = 0;
  double max = 100;

  @override
  FutureOr<void> afterFirstLayout(BuildContext context) {
    setState(() {
      min = widget.min;
      max = widget.max;
      values = RangeValues(min, max);
    });
  }

  void _handleOnMinChanged(double v) {
    runWorkflows(
      node: widget.state.node,
      trigger: Trigger.onMinValueChanged,
      context: context,
      loop: widget.state.loop,
      inputValue: v,
    );
  }

  void _handleOnMaxChanged(double v) {
    runWorkflows(
      node: widget.state.node,
      trigger: Trigger.onMaxValueChanged,
      context: context,
      loop: widget.state.loop,
      inputValue: v,
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuildersForNodes(
      node: widget.state.node,
      builder: (context, state, stylesState, themeMode) {
        final activeColor = widget.fill.getColor(
          context,
          widget.state.loop,
        );

        return IgnorePointer(
          ignoring: !state.forPlay,
          child: RangeSlider(
            min: min,
            max: max,
            values: values,
            onChanged: (newValues) {
              _handleOnMinChanged(newValues.end);
              _handleOnMaxChanged(newValues.end);
              setState(() {
                values = newValues;
              });
            },
            activeColor: activeColor,
          ),
        );
      },
    );
  }
}
