import 'package:designflow_widgets/designflow_widgets.dart';
import 'package:flutter/material.dart';

class OpenWMaterialSlider extends StatefulWidget {
  const OpenWMaterialSlider({
    super.key,
    required this.state,
    required this.fill,
    required this.min,
    required this.max,
    required this.formatType,
  });

  final WidgetState state;
  final double min;
  final double max;
  final FFill fill;
  final FormatType formatType;

  @override
  State<OpenWMaterialSlider> createState() => _OpenWMaterialSliderState();
}

class _OpenWMaterialSliderState extends State<OpenWMaterialSlider> {
  late double value;

  @override
  void initState() {
    super.initState();
    value = widget.min;
  }

  double convertToMinutes(double number) {
    // Define the maximum values for the ranges
    const double initialRangeMax = 99;
    const double targetRangeMax = 59;

    // Calculate the conversion factor
    const double conversionFactor = targetRangeMax / initialRangeMax;

    // Convert the number
    double convertedNumber = number * conversionFactor;

    return convertedNumber;
  }

  String _formatForTime(double value) {
    var hours = '${value ~/ 100}';
    if (hours.length == 1) {
      hours = '0$hours';
    }
    var minutes = '${convertToMinutes(value % 100).toInt()}';
    if (minutes.length == 1) {
      minutes = '0$minutes';
    }
    return '$hours:$minutes';
  }

  void _handleOnChanged(double v) {
    final value = widget.formatType == FormatType.hourAndMinutes
        ? _formatForTime(v)
        : widget.formatType == FormatType.int
            ? v.toInt()
            : widget.formatType == FormatType.limitedDouble
                ? double.parse(v.toStringAsFixed(2))
                : v;
    runWorkflows(
      node: widget.state.node,
      trigger: Trigger.onChange,
      context: context,
      loop: widget.state.loop,
      inputValue: value,
    );
    setState(() {
      this.value = v;
    });
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuildersForNodes(
      node: widget.state.node,
      builder: (context, state, stylesState, themeMode) {
        final activeColor = widget.fill.getColor(
          context,
          widget.state.loop,
        );

        return IgnorePointer(
          ignoring: !state.forPlay,
          child: Slider(
            min: widget.min,
            max: widget.max,
            value: value,
            onChanged: (value) => _handleOnChanged(value),
            activeColor: activeColor,
            label: value.toString(),
          ),
        );
      },
    );
  }
}
